@@ -5,5 +5,5 @@ if [ -z $1 ]; then
 fi
 
 date
-scp vmm/vmm.elf cmd/dbgsh bios/out/bios.bin $1
+scp vmm.elf bios.bin $1
 
