@@ -459,9 +459,9 @@ acpi_pm1_sleep (u32 v)
 		else
 			m[i] = false;
 	}
-	if (m[5]) {
+	if (m[5] || m[4]) {
 		ACPI_DBG("shutdown\n");
-		vm_shutdown();
+		vm_shutdown(m[5] ? 5 : 4);
 		/* not reached */
 	}
 
@@ -472,8 +472,7 @@ acpi_pm1_sleep (u32 v)
 		return false;
 #endif /* DISABLE_SLEEP */
 #endif /* ACPI_DSDT */
-
-	ACPI_DBG("Enter to S3 or S4\n");
+	ACPI_DBG("Enter to S2 or S3\n");
 	old_waking_vector = 0;
 	for (i = 0; i < 4; i++) {
 		if (!facs_addr[i])
@@ -782,17 +781,19 @@ acpi_reset (void)
 }
 
 void
-acpi_poweroff (void)
+acpi_poweroff (u8 state)
 {
 	u32 data, typx;
 
 	if (!pm1a_cnt_found)
 		return;
-	if (!acpi_sleep[5].valid)
+	if (state < 4 || state > 5)
+		return;
+	if (!acpi_sleep[state].valid)
 		return;
 
 	ACPI_DBG("acpi_poweroff\n");
-	typx = acpi_sleep[5].type[0] << PM1_CNT_SLP_TYPX_SHIFT;
+	typx = acpi_sleep[state].type[0] << PM1_CNT_SLP_TYPX_SHIFT;
 
 	asm_inl (pm1a_cnt_ioaddr, &data);
 	data &= ~PM1_CNT_SLP_TYPX_MASK;
@@ -801,7 +802,7 @@ acpi_poweroff (void)
 	asm_outl (pm1a_cnt_ioaddr, data);
 
 	if (pm1b_cnt_found) {
-		typx = acpi_sleep[5].type[1] << PM1_CNT_SLP_TYPX_SHIFT;
+		typx = acpi_sleep[state].type[1] << PM1_CNT_SLP_TYPX_SHIFT;
 		asm_inl (pm1b_cnt_ioaddr, &data);
 		data &= ~PM1_CNT_SLP_TYPX_MASK;
 		data |= typx & PM1_CNT_SLP_TYPX_MASK;
