@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011-2013 Yuichi Watanabe
+ * Copyright (c) 2011-2014 Yuichi Watanabe
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -32,11 +32,11 @@
 
 #include <core/cpu.h>
 #include <core/types.h>
-#include <io/io.h>
+#include <io.h>
 #include "pcpu.h"
 
 #define apic_g2v(gphys) \
-	(void *)((char *)apic_virt_base + ((gphys) - APIC_BASE_ADDRESS))
+	(void *)((char *)apic_virt_base + ((gphys) - APIC_BASE))
 
 extern void *apic_virt_base;
 
@@ -52,15 +52,19 @@ apic_read32(phys_t offset)
 	return read32(apic_virt_base + offset);
 }
 
-void apic_send_init(apic_id_t dest);
+bool apic_available(void);
 void apic_wait_for_idle(void);
 apic_id_t apic_read_id(void);
-void apic_start_ap(phys_t apinit_addr);
+void apic_start_ap (u8 addr);
 void apic_broadcast_nmi(void);
 bool apic_check_apic_interrupt(vector_t vector);
 void apic_write_eoi(void);
 u32 apic_read_icrh(void);
 void apic_send_ipi(vector_t vector, apic_id_t dest);
+void apic_send_sipi(vector_t vector, apic_id_t dest);
+void apic_send_init(apic_id_t dest);
+void apic_disable(void);
 void apic_maskall(void);
+void apic_set_priority(vector_t priority);
 
 #endif
