@@ -1,3 +1,6 @@
+/*
+ * From BitVisor 1.0.1 core/osloader.h
+ */
 /*
  * Copyright (c) 2007, 2008 University of Tsukuba
  * All rights reserved.
@@ -34,6 +37,7 @@
 #define _CORE_GBIOSLOADER_H
 
 #include <core/types.h>
+#include <core/vmmerr.h>
 
 void gbios_save (u32 start, u32 size);
 vmmerr_t gbios_load (void);
