@@ -26,6 +26,9 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  * POSSIBILITY OF SUCH DAMAGE.
  */
+/*
+ * Copyright (c) 2014 Yuichi Watanabe
+ */
 
 #include "asm.h"
 #include "constants.h"
@@ -38,6 +41,7 @@
 #include "panic.h"
 #include "printf.h"
 #include "vm.h"
+#include "uefi.h"
 
 struct call_iopass_data {
 	struct vm *vcpu0;
@@ -98,14 +102,35 @@ set_iofunc (u32 port, iofunc_t func)
 	return old;
 }
 
+static void
+io_io_delayed_init(void)
+{
+	iofunc_t func;
+	bool pass;
+	u32 i;
+
+	for (i = 0; i < NUM_OF_IOPORT; i++) {
+		func = current->vcpu0->io.iofunc[i];
+		if (func == do_iopass_default) {
+			pass = true;
+		} else {
+			pass = false;
+		}
+		current->vmctl.extern_iopass(current, i, pass);
+	}
+}
+
 static void
 io_io_init (void)
 {
 	u32 i;
 
-	if (current->vbsp)
+	if (current->vbsp) {
 		for (i = 0; i < NUM_OF_IOPORT; i++)
 			set_iofunc (i, do_io_nothing);
+	} else if (vm_get_id() == 0 && uefi_booted) {
+		io_io_delayed_init();
+	}
 }
 
 enum ioact
