@@ -142,8 +142,16 @@ copy_cmdline (void)
 		if (vaddr == NULL) {
 			panic("Failed to map multiboot_info.cmdline.");
 		}
-		param_save(vaddr);
+		param_save(vaddr, CMDLINE_SIZE);
 		unmapmem(vaddr, CMDLINE_SIZE);
+	} else if (uefi_vmm_data.param_paddr) {
+		vaddr = mapmem (MAPMEM_HPHYS, uefi_vmm_data.param_paddr,
+				uefi_vmm_data.param_size);
+		if (vaddr == NULL) {
+			panic("Failed to map param.");
+		}
+		param_save(vaddr, uefi_vmm_data.param_size);
+		unmapmem(vaddr, uefi_vmm_data.param_size);
 	}
 }
 
@@ -160,7 +168,12 @@ copy_gbios (void)
 		}
 		printf("Guest BIOS found. size 0x%x\n", mod->mod_end - mod->mod_start);
 		gbios_save(mod->mod_start, mod->mod_end - mod->mod_start);
-		unmapmem(mod, sizeof(mod));;
+		unmapmem(mod, sizeof(mod));
+	} else if (uefi_vmm_data.module_paddr) {
+		printf("Guest BIOS found. size 0x%llx\n",
+		       uefi_vmm_data.module_size);
+		gbios_save(uefi_vmm_data.module_paddr,
+			   uefi_vmm_data.module_size);
 	} else {
 		printf("Guest BIOS not found.\n");
 	}
