@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2010-2012 Yuichi Watanabe
+ * Copyright (c) 2010-2014 Yuichi Watanabe
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -47,9 +47,13 @@ enum param_state {
 static char saved_cmdline[CMDLINE_SIZE];
 
 void
-param_save(const char *cmdline)
+param_save(const char *cmdline, size_t len)
 {
-	snprintf(saved_cmdline, CMDLINE_SIZE, "%s", cmdline);
+	len++;
+	if (len > CMDLINE_SIZE) {
+		len = CMDLINE_SIZE;
+	}
+	snprintf(saved_cmdline, len, "%s", cmdline);
 	printf("VMM params: %s\n", saved_cmdline);
 }
 
@@ -76,6 +80,8 @@ param_get_str(const char *name, int number, char *buf, size_t len)
 		case '\0':
 		case ' ':
 		case '\t':
+		case '\r':
+		case '\n':
 			switch (state) {
 			case STATE_NAME:
 				if (na_len == name_len &&
