@@ -26,34 +26,25 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  * POSSIBILITY OF SUCH DAMAGE.
  */
-/*
- * Copyright (c) 2010-2013 Yuichi Watanabe
- */
 
-#include <core/cpu.h>
-#include <core/initfunc.h>
-#include <core/printf.h>
-#include <core/string.h>
 #include "asm.h"
+#include "assert.h"
 #include "constants.h"
+#include "cpu.h"
 #include "current.h"
+#include "initfunc.h"
+#include "int.h"
+#include "localapic.h"
 #include "mm.h"
 #include "panic.h"
 #include "pcpu.h"
+#include "printf.h"
+#include "string.h"
 #include "svm_init.h"
-#include "svm_np.h"
+#include "svm_paging.h"
 #include "svm_vmcb.h"
-#include <core/types.h>
-
-/* #define SVM_DEBUG */
-#ifdef SVM_DEBUG
-#define SVM_DBG(...)						\
-	do {							\
-		printf(__VA_ARGS__);				\
-	} while (0)
-#else
-#define SVM_DBG(...)
-#endif
+#include "sx_handler.h"
+#include "types.h"
 
 bool
 svm_available (void)
@@ -63,12 +54,12 @@ svm_available (void)
 
 	asm_cpuid (CPUID_EXT_0, 0, &a, &b, &c, &d);
 	if (a < CPUID_EXT_1) {
-		SVM_DBG("CPU%d does not support SVM.\n", get_cpu_id());
+		/* printf ("SVM is not available.\n"); */
 		return false;
 	}
 	asm_cpuid (CPUID_EXT_1, 0, &a, &b, &c, &d);
 	if (!(c & CPUID_EXT_1_ECX_SVM_BIT)) {
-		SVM_DBG("CPU%d does not support SVM.\n", get_cpu_id());
+		/* printf ("SVM is not available.\n"); */
 		return false;
 	}
 	asm_rdmsr64 (MSR_AMD_VM_CR, &tmp);
@@ -76,25 +67,19 @@ svm_available (void)
 		return true;	/* SVM is allowed */
 	asm_cpuid (CPUID_EXT_0, 0, &a, &b, &c, &d);
 	if (a < CPUID_EXT_A) {
-		printf ("SVM is disabled on CPU%d.\n", get_cpu_id());
+		printf ("SVM is disabled.\n");
 		return false;
 	}
 	asm_cpuid (CPUID_EXT_A, 0, &a, &b, &c, &d);
 	if (!(d & CPUID_EXT_A_EDX_SVM_LOCK_BIT)) {
-		printf ("SVM is disabled by BIOS on CPU%d.\n", get_cpu_id());
+		printf ("SVM is disabled at BIOS.\n");
 		return false;
 	} else {
-		printf ("SVM is disabled with a key on CPU%d.\n", get_cpu_id());
+		printf ("SVM is disabled with a key.\n");
 		return false;
 	}
 }
 
-static bool
-svm_nested_paging_available (void)
-{
-	return true;
-}
-
 static void
 svm_seg_reset (struct vmcb *p)
 {
@@ -144,36 +129,44 @@ void
 svm_reset (void)
 {
 	svm_seg_reset (current->u.svm.vi.vmcb);
+	if (*current->u.svm.cr0 & CR0_PG_BIT) {
+		*current->u.svm.cr0 = CR0_ET_BIT;
+		svm_paging_pg_change ();
+	} else {
+		*current->u.svm.cr0 = CR0_ET_BIT;
+	}
+	current->u.svm.vi.vmcb->cr0 = svm_paging_apply_fixed_cr0 (CR0_ET_BIT);
+	current->u.svm.vi.vmcb->cr2 = 0;
+	*current->u.svm.cr3 = 0;
+	*current->u.svm.cr4 = 0;
+	current->u.svm.vi.vmcb->cr4 = svm_paging_apply_fixed_cr4 (0);
+	current->u.svm.vi.vmcb->efer = MSR_IA32_EFER_SVME_BIT;
+	current->u.svm.lme = 0;
+	svm_msr_update_lma ();
+	svm_paging_updatecr3 ();
+	svm_paging_flush_guest_tlb ();
 }
 
 static void
 svm_vmcb_init (void)
 {
 	struct vmcb *p;
-	vmmerr_t err;
-	void *page;
 
-	err = alloc_page (&page, &current->u.svm.vi.vmcb_phys);
-	if (err) {
-		panic("svm_vmcb_init: Failed to allocate memory for VMCB");
-	}
-	current->u.svm.vi.vmcb = page;
-	err = alloc_pages (&current->u.svm.io.iobmp,
-			   &current->u.svm.io.iobmp_phys, 3);
-	if (err) {
-		panic("svm_vmcb_init: Failed to allocate memory for IO bitmap");
-	}
-	err = alloc_pages (&current->u.svm.msr.msrbmp,
-			   &current->u.svm.msr.msrbmp_phys, 2);
-	if (err) {
-		panic("svm_vmcb_init: Failed to allocate memory for MSR bitmap");
-	}
+	current->u.svm.saved_vmcb = NULL;
+	alloc_page ((void **)&current->u.svm.vi.vmcb,
+		    &current->u.svm.vi.vmcb_phys);
+	alloc_pages (&current->u.svm.io.iobmp,
+		     &current->u.svm.io.iobmp_phys, 3);
+	alloc_pages (&current->u.svm.msr.msrbmp,
+		     &current->u.svm.msr.msrbmp_phys, 2);
 	memset (current->u.svm.io.iobmp, 0xFF, PAGESIZE * 3);
 	memset (current->u.svm.msr.msrbmp, 0xFF, PAGESIZE * 2);
+	/* passthrough writing to PATCH_LOADER MSR (0xC0010020) */
+	((u8 *)current->u.svm.msr.msrbmp)[0x1008] &= ~2;
 	p = current->u.svm.vi.vmcb;
 	memset (p, 0, PAGESIZE);
-	p->intercept_read_cr = ~4;
-	p->intercept_write_cr = ~4;
+	p->intercept_read_cr = ~0x104;
+	p->intercept_write_cr = ~0x104;
 	p->intercept_exception = 0x4000;
 	p->intercept_intr = 1;
 	p->intercept_nmi = 1;
@@ -186,7 +179,7 @@ svm_vmcb_init (void)
 	p->intercept_shutdown = 1;
 	p->intercept_vmrun = 1;
 	p->intercept_vmmcall = 1;
-	p->v_ign_tpr = 1;
+	p->intercept_cpuid = 1;
 	p->iopm_base_pa = current->u.svm.io.iobmp_phys;
 	p->msrpm_base_pa = current->u.svm.msr.msrbmp_phys;
 	p->guest_asid = 1;	/* FIXME */
@@ -196,15 +189,6 @@ svm_vmcb_init (void)
 	p->efer = MSR_IA32_EFER_SVME_BIT;
 	p->cr0 = CR0_PG_BIT;
 	p->rflags = RFLAGS_ALWAYS1_BIT;
-	if (false && svm_nested_paging_available ()) {
-		/* FIXME: Nested paging */
-		p->np_enable = 1;
-		p->intercept_invlpg = 0;
-		p->intercept_exception &= ~0x4000;
-		p->intercept_read_cr &= ~8;
-		p->intercept_write_cr &= ~8;
-		asm_rdmsr64 (0x277, &p->g_pat);
-	}
 }
 
 void
@@ -213,10 +197,8 @@ svm_vminit (void)
 	current->u.svm.np = NULL;
 	svm_vmcb_init ();
 	/* svm_msr_init (); */
-	if (current->u.svm.vi.vmcb->np_enable)
-		svm_np_init ();
-	else
-		cpu_mmu_spt_init ();
+	svm_paging_init ();
+	call_initfunc ("vcpu");
 }
 
 void
@@ -231,27 +213,59 @@ svm_init (void)
 	u64 tmp;
 	void *v;
 	ulong efer;
-	vmmerr_t err;
+	u32 a, b, c, d;
 
 	asm_rdmsr (MSR_IA32_EFER, &efer);
 	efer |= MSR_IA32_EFER_SVME_BIT;
 	asm_wrmsr (MSR_IA32_EFER, efer);
 	asm_rdmsr64 (MSR_AMD_VM_CR, &tmp);
-	tmp |= MSR_AMD_VM_CR_DIS_A20M_BIT;
+	tmp |= MSR_AMD_VM_CR_DIS_A20M_BIT | MSR_AMD_VM_CR_R_INIT_BIT;
 	asm_wrmsr64 (MSR_AMD_VM_CR, tmp);
+	set_int_handler (EXCEPTION_SX, sx_handler);
 	/* FIXME: size of a host state area is undocumented */
-	err = alloc_page(&v, &p);
-	if (err) {
-		panic("Failed to allocate a host state area of SVM.");
-	}
+	alloc_page (&v, &p);
 	currentcpu->svm.hsave = v;
 	currentcpu->svm.hsave_phys = p;
 	asm_wrmsr64 (MSR_AMD_VM_HSAVE_PA, p);
-	err = alloc_page (&v, &p);
-	if (err) {
-		panic("Failed to allocate a VMCB.");
-	}
+	alloc_page (&v, &p);
 	memset (v, 0, PAGESIZE);
 	currentcpu->svm.vmcbhost = v;
 	currentcpu->svm.vmcbhost_phys = p;
+	/* CPUID_EXT_A is available if svm_available() returns true */
+	asm_cpuid (CPUID_EXT_A, 0, &a, &b, &c, &d);
+	if (d & CPUID_EXT_A_EDX_FLUSH_BY_ASID_BIT)
+		currentcpu->svm.flush_by_asid = true;
+	else
+		currentcpu->svm.flush_by_asid = false;
+}
+
+void
+svm_enable_resume (void)
+{
+	void *virt;
+
+	ASSERT (!current->u.svm.saved_vmcb);
+	alloc_page (&virt, NULL);
+	current->u.svm.saved_vmcb = virt;
+	memcpy (current->u.svm.saved_vmcb, current->u.svm.vi.vmcb, PAGESIZE);
+	spinlock_lock (&currentcpu->suspend_lock);
+}
+
+void
+svm_resume (void)
+{
+	u64 tmp;
+	ulong efer;
+
+	ASSERT (current->u.svm.saved_vmcb);
+	asm_rdmsr (MSR_IA32_EFER, &efer);
+	efer |= MSR_IA32_EFER_SVME_BIT;
+	asm_wrmsr (MSR_IA32_EFER, efer);
+	asm_rdmsr64 (MSR_AMD_VM_CR, &tmp);
+	tmp |= MSR_AMD_VM_CR_DIS_A20M_BIT | MSR_AMD_VM_CR_R_INIT_BIT;
+	asm_wrmsr64 (MSR_AMD_VM_CR, tmp);
+	asm_wrmsr64 (MSR_AMD_VM_HSAVE_PA, currentcpu->svm.hsave_phys);
+	memcpy (current->u.svm.vi.vmcb, current->u.svm.saved_vmcb, PAGESIZE);
+	spinlock_init (&currentcpu->suspend_lock);
+	spinlock_lock (&currentcpu->suspend_lock);
 }
