@@ -26,30 +26,29 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  * POSSIBILITY OF SUCH DAMAGE.
  */
+
 /*
  * Copyright (c) 2010-2013 Yuichi Watanabe
  */
 
-#include <common/common.h>
-#include <core/initfunc.h>
-#include <core/spinlock.h>
-#include <core/string.h>
 #include "apic.h"
-#include "apic_pass.h"
 #include "current.h"
+#include "initfunc.h"
 #include "mm.h"
+#include "spinlock.h"
+#include "string.h"
+#include "types.h"
 #include "pcpu.h"
-#include <core/types.h>
 #include "vcpu.h"
 
 /* call func with every vcpu */
 /* return immediately if func returns true */
 /* q is a pointer for any purpose */
 void
-vcpu_list_foreach (struct vm *vm, bool (*func) (struct vcpu *p, void *q),
-		   void *q)
+vcpu_list_foreach (bool (*func) (struct vcpu *p, void *q), void *q)
 {
 	struct vcpu *p, *pn;
+	struct vm *vm = current->vm;
 
 	spinlock_lock(&vm->vcpu_list_lock);
 	LIST2_FOREACH_DELETABLE(vm->vcpu_list, vcpu_list, p, pn) {
@@ -81,7 +80,7 @@ find_vcpu_with_apic_id (struct vm *vm, apic_id_t apic_id)
 #include <core/printf.h>
 
 struct vcpu *
-new_vcpu (struct vm *vm, bool vbsp)
+vcpu_new (struct vm *vm, bool vbsp)
 {
 	current = (struct vcpu *)alloc(sizeof (struct vcpu));
 	if (current == NULL) {
@@ -89,11 +88,9 @@ new_vcpu (struct vm *vm, bool vbsp)
 	}
 	memset(current, 0, sizeof (struct vcpu));
 	current->vbsp = vbsp;
-	current->pcpu = currentcpu;
 	current->apic_id = currentcpu->apic_id;
 	current->vm = vm;
-
-	int_init_hook();
+	current->vcpu0 = vm;
 
 	spinlock_lock(&vm->vcpu_list_lock);
 	LIST2_ADD(vm->vcpu_list, vcpu_list, current);
@@ -125,30 +122,13 @@ vcpu_reset(void)
 	current->vmctl.write_general_reg (GENERAL_REG_RBP, 0);
 	current->vmctl.write_general_reg (GENERAL_REG_RSI, 0);
 	current->vmctl.write_general_reg (GENERAL_REG_RDI, 0);
-	current->vmctl.write_general_reg (GENERAL_REG_R8, 0);
-	current->vmctl.write_general_reg (GENERAL_REG_R9, 0);
-	current->vmctl.write_general_reg (GENERAL_REG_R10, 0);
-	current->vmctl.write_general_reg (GENERAL_REG_R11, 0);
-	current->vmctl.write_general_reg (GENERAL_REG_R12, 0);
-	current->vmctl.write_general_reg (GENERAL_REG_R13, 0);
-	current->vmctl.write_general_reg (GENERAL_REG_R14, 0);
-	current->vmctl.write_general_reg (GENERAL_REG_R15, 0);
-	current->vmctl.write_flags (RFLAGS_ALWAYS1_BIT);
-	current->vmctl.write_realmode_seg (SREG_ES, 0);
 	current->vmctl.write_realmode_seg (SREG_CS, 0xf000);
-	current->vmctl.write_realmode_seg (SREG_SS, 0);
-	current->vmctl.write_realmode_seg (SREG_DS, 0);
-	current->vmctl.write_realmode_seg (SREG_FS, 0);
-	current->vmctl.write_realmode_seg (SREG_GS, 0);
-	current->vmctl.write_control_reg (CONTROL_REG_CR0, CR0_ET_BIT);
-	current->vmctl.write_control_reg (CONTROL_REG_CR2, 0);
-	current->vmctl.write_control_reg (CONTROL_REG_CR3, 0);
-	current->vmctl.write_control_reg (CONTROL_REG_CR4, 0);
 	current->vmctl.write_ip (0xfff0);
+	current->vmctl.write_flags (RFLAGS_ALWAYS1_BIT);
 	current->vmctl.write_idtr (0, 0x3FF);
 	current->vmctl.write_gdtr (0, 0xFFFF);
 
 	if (!current->vbsp) {
-		apic_enter_wait_for_sipi_state();
+		current->vmctl.init_signal ();
 	}
 }
