@@ -26,32 +26,26 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  * POSSIBILITY OF SUCH DAMAGE.
  */
-/*
- * Copyright (c) 2010-2014 Yuichi Watanabe
- */
 
-#include <core/initfunc.h>
-#include <core/cpu.h>
-#include <core/printf.h>
-#include <core/string.h>
 #include "asm.h"
+#include "assert.h"
 #include "constants.h"
-#include "cpu_mmu_spt.h"
 #include "current.h"
+#include "initfunc.h"
 #include "panic.h"
 #include "mm.h"
 #include "pcpu.h"
-#include "vt_internal.h"
+#include "printf.h"
+#include "string.h"
+#include "vt_exitreason.h"
+#include "vt_init.h"
+#include "vt_paging.h"
+#include "vt_panic.h"
+#include "vt_regs.h"
 
-/* #define VT_DEBUG */
-#ifdef VT_DEBUG
-#define VT_DBG(...)						\
-	do {							\
-		printf(__VA_ARGS__);				\
-	} while (0)
-#else
-#define VT_DBG(...)
-#endif
+/* #define DISABLE_VPID */
+/* #define DISABLE_EPT */
+/* #define DISABLE_UNRESTRICTED_GUEST */
 
 /* Check whether VMX is usable
    Return value:
@@ -68,24 +62,28 @@ check_vmx (void)
 	if (c & CPUID_1_ECX_VMX_BIT) {
 		/* VMX operation is supported. */
 	} else {
-		VT_DBG("CPU%d does not support VMX operation.\n", get_cpu_id());
+		/* printf ("VMX operation is not supported.\n"); */
 		return -1;
 	}
 
 	/* 19.7 ENABLING AND ENTERING VMX OPERATION */
+msr_enable_loop:
 	asm_rdmsr64 (MSR_IA32_FEATURE_CONTROL, &tmp);
 	if (tmp & MSR_IA32_FEATURE_CONTROL_LOCK_BIT) {
 		if (tmp & MSR_IA32_FEATURE_CONTROL_VMXON_BIT) {
-			/* VMX operation is allowed by bios. */
+			/* VMXON is enabled. */
 		} else {
-			printf("VMX operation is not allowed by BIOS.");
+			printf ("VMXON is disabled.\n");
 			return -1;
 		}
 	} else {
+		printf ("Trying to enable VMXON.\n");
 		tmp |= MSR_IA32_FEATURE_CONTROL_VMXON_BIT;
 		tmp |= MSR_IA32_FEATURE_CONTROL_LOCK_BIT;
 		asm_wrmsr64 (MSR_IA32_FEATURE_CONTROL, tmp);
+		goto msr_enable_loop;
 	}
+
 	return 0;
 }
 
@@ -96,13 +94,13 @@ vt_available (void)
 }
 
 void
-vt_vmx_init (void)
+vt__vmx_init (void)
 {
 	void *v;
 	u64 p;
 
 	if (alloc_page (&v, &p) < 0)
-		panic ("vt_vmx_init: alloc_page failed.");
+		panic ("Fatal error: vt__vmx_init: alloc_page failed.");
 	currentcpu->vt.vmxon_region_virt = v;
 	currentcpu->vt.vmxon_region_phys = p;
 }
@@ -115,7 +113,7 @@ vt_vmx_init (void)
    vmcs_revision_identifier: VMCS revision identifier
 */
 void
-vt_vmxon (void)
+vt__vmxon (void)
 {
 	ulong cr0_0, cr0_1, cr4_0, cr4_1;
 	ulong cr0, cr4;
@@ -151,27 +149,40 @@ vt_vmxon (void)
 	asm_vmxon (&currentcpu->vt.vmxon_region_phys);
 }
 
-void
-vt_reset (void)
+static void
+vpid_init (void)
 {
-	struct vt_intr_data *vid = &current->u.vt.intr;
+#ifndef DISABLE_VPID
+	u64 ept_vpid_cap;
 
-	vid->intr_info.v = 0;
-	asm_vmwrite (VMCS_GUEST_INTERRUPTIBILITY_STATE, 0);
-	asm_vmwrite (VMCS_GUEST_ACTIVITY_STATE, 0);
-	asm_vmwrite (VMCS_GUEST_PENDING_DEBUG_EXCEPTIONS, 0);
+	asm_rdmsr64 (MSR_IA32_VMX_EPT_VPID_CAP, &ept_vpid_cap);
+	if (!(ept_vpid_cap & MSR_IA32_VMX_EPT_VPID_CAP_INVVPID_BIT))
+		return;
+	if (!(ept_vpid_cap &
+	      MSR_IA32_VMX_EPT_VPID_CAP_INVVPID_SINGLE_CONTEXT_BIT))
+		return;
+	current->u.vt.vpid = 1; /* FIXME: VPID 1 only */
+#endif
 }
 
 static void
-vt_vmptrld (u64 ptr)
+ept_init (void)
 {
-	if (currentcpu->vt.vmcs_region_phys == ptr)
+#ifndef DISABLE_EPT
+	u64 ept_vpid_cap;
+
+	asm_rdmsr64 (MSR_IA32_VMX_EPT_VPID_CAP, &ept_vpid_cap);
+	if (!(ept_vpid_cap & MSR_IA32_VMX_EPT_VPID_CAP_PAGEWALK_LENGTH_4_BIT))
 		return;
-	if (currentcpu->vt.vmcs_region_phys != 0)
-		asm_vmptrst (&currentcpu->vt.vmcs_region_phys);
-	currentcpu->vt.vmcs_region_phys = ptr;
-	if (currentcpu->vt.vmcs_region_phys != 0)
-		asm_vmptrld (&currentcpu->vt.vmcs_region_phys);
+	if (!(ept_vpid_cap & MSR_IA32_VMX_EPT_VPID_CAP_EPTSTRUCT_WB_BIT))
+		return;
+	current->u.vt.ept_available = true;
+	if (!(ept_vpid_cap & MSR_IA32_VMX_EPT_VPID_CAP_INVEPT_BIT))
+		return;
+	if (!(ept_vpid_cap & MSR_IA32_VMX_EPT_VPID_CAP_INVEPT_ALL_CONTEXT_BIT))
+		return;
+	current->u.vt.invept_available = true;
+#endif
 }
 
 /* Initialize VMCS region
@@ -179,7 +190,7 @@ vt_vmptrld (u64 ptr)
    vmcs_revision_identifier: VMCS revision identifier
  */
 static void
-vt_vmcs_init (void)
+vt__vmcs_init (void)
 {
 	u32 *p;
 	struct regs_in_vmcs host_riv, guest_riv;
@@ -189,16 +200,26 @@ vt_vmcs_init (void)
 	u32 entry_ctls_or, entry_ctls_and;
 	ulong sysenter_cs, sysenter_esp, sysenter_eip;
 	ulong exitctl64;
-	vmmerr_t err;
+	ulong exitctl_efer = 0, entryctl_efer = 0;
+	u64 host_efer;
+	u32 procbased_ctls2_or, procbased_ctls2_and = 0;
+	ulong procbased_ctls2 = 0;
 
+	current->u.vt.first = true;
 	current->u.vt.io.iobmpflag = false;
-	err = alloc_page (&current->u.vt.vi.vmcs_region_virt,
-			  &current->u.vt.vi.vmcs_region_phys);
-	if (err) {
-		panic("Failed to allocate VMCS.");
-	}
-	memset(current->u.vt.vi.vmcs_region_virt, 0, PAGE_SIZE);
-	current->u.vt.intr.intr_info.s.valid = INTR_INFO_VALID_INVALID;
+	current->u.vt.saved_vmcs = NULL;
+	current->u.vt.vpid = 0;
+	current->u.vt.ept = NULL;
+	current->u.vt.ept_available = false;
+	current->u.vt.invept_available = false;
+	current->u.vt.unrestricted_guest_available = false;
+	current->u.vt.unrestricted_guest = false;
+	current->u.vt.save_load_efer_enable = false;
+	current->u.vt.exint_pass = true;
+	current->u.vt.exint_pending = false;
+	alloc_page (&current->u.vt.vi.vmcs_region_virt,
+		    &current->u.vt.vi.vmcs_region_phys);
+	current->u.vt.intr.vmcs_intr_info.s.valid = INTR_INFO_VALID_INVALID;
 
 	p = current->u.vt.vi.vmcs_region_virt;
 	*p = currentcpu->vt.vmcs_revision_identifier; /* write VMCS revision
@@ -218,6 +239,38 @@ vt_vmcs_init (void)
 	asm_rdmsr (MSR_IA32_SYSENTER_CS, &sysenter_cs);
 	asm_rdmsr (MSR_IA32_SYSENTER_ESP, &sysenter_esp);
 	asm_rdmsr (MSR_IA32_SYSENTER_EIP, &sysenter_eip);
+	if (procbased_ctls_and &
+	    VMCS_PROC_BASED_VMEXEC_CTL_ACTIVATECTLS2_BIT) {
+		asm_rdmsr32 (MSR_IA32_VMX_PROCBASED_CTLS2,
+			     &procbased_ctls2_or, &procbased_ctls2_and);
+		/* procbased_ctls2_or is always zero */
+		procbased_ctls2 = 0;
+		if (procbased_ctls2_and &
+		    VMCS_PROC_BASED_VMEXEC_CTL2_ENABLE_VPID_BIT)
+			vpid_init ();
+		if (current->u.vt.vpid)
+			procbased_ctls2 |=
+				VMCS_PROC_BASED_VMEXEC_CTL2_ENABLE_VPID_BIT;
+		if ((procbased_ctls2_and &
+		     VMCS_PROC_BASED_VMEXEC_CTL2_ENABLE_EPT_BIT) &&
+		    (exit_ctls_and & VMCS_VMEXIT_CTL_SAVE_IA32_PAT_BIT) &&
+		    (exit_ctls_and & VMCS_VMEXIT_CTL_LOAD_IA32_PAT_BIT) &&
+		    (entry_ctls_and & VMCS_VMENTRY_CTL_LOAD_IA32_PAT_BIT))
+			ept_init ();
+#ifndef DISABLE_UNRESTRICTED_GUEST
+		if (procbased_ctls2_and &
+		    VMCS_PROC_BASED_VMEXEC_CTL2_UNRESTRICTED_GUEST_BIT)
+			current->u.vt.unrestricted_guest_available = true;
+#endif
+	}
+	if ((exit_ctls_and & VMCS_VMEXIT_CTL_SAVE_IA32_EFER_BIT) &&
+	    (exit_ctls_and & VMCS_VMEXIT_CTL_LOAD_IA32_EFER_BIT) &&
+	    (entry_ctls_and & VMCS_VMENTRY_CTL_LOAD_IA32_EFER_BIT)) {
+		current->u.vt.save_load_efer_enable = true;
+		exitctl_efer |= VMCS_VMEXIT_CTL_SAVE_IA32_EFER_BIT;
+		exitctl_efer |= VMCS_VMEXIT_CTL_LOAD_IA32_EFER_BIT;
+		entryctl_efer |= VMCS_VMENTRY_CTL_LOAD_IA32_EFER_BIT;
+	}
 
 	/* get current information */
 	vt_get_current_regs_in_vmcs (&host_riv);
@@ -226,6 +279,9 @@ vt_vmcs_init (void)
 		VMCS_VMEXIT_CTL_HOST_ADDRESS_SPACE_SIZE_BIT;
 
 	/* initialize VMCS fields */
+	/* 16-Bit Control Field */
+	if (current->u.vt.vpid)
+		asm_vmwrite (VMCS_VPID, current->u.vt.vpid);
 	/* 16-Bit Guest-State Fields */
 	asm_vmwrite (VMCS_GUEST_ES_SEL, guest_riv.es.sel);
 	asm_vmwrite (VMCS_GUEST_CS_SEL, guest_riv.cs.sel);
@@ -244,53 +300,49 @@ vt_vmcs_init (void)
 	asm_vmwrite (VMCS_HOST_GS_SEL, host_riv.gs.sel);
 	asm_vmwrite (VMCS_HOST_TR_SEL, host_riv.tr.sel);
 	/* 64-Bit Control Fields */
-	asm_vmwrite (VMCS_ADDR_IOBMP_A, 0xFFFFFFFF);
-	asm_vmwrite (VMCS_ADDR_IOBMP_A_HIGH, 0xFFFFFFFF);
-	asm_vmwrite (VMCS_ADDR_IOBMP_B, 0xFFFFFFFF);
-	asm_vmwrite (VMCS_ADDR_IOBMP_B_HIGH, 0xFFFFFFFF);
-	asm_vmwrite (VMCS_ADDR_MSRBMP, 0xFFFFFFFF);
-	asm_vmwrite (VMCS_ADDR_MSRBMP_HIGH, 0xFFFFFFFF);
-	asm_vmwrite (VMCS_VMEXIT_MSRSTORE_ADDR, 0);
-	asm_vmwrite (VMCS_VMEXIT_MSRSTORE_ADDR_HIGH, 0);
-	asm_vmwrite (VMCS_VMEXIT_MSRLOAD_ADDR, 0);
-	asm_vmwrite (VMCS_VMEXIT_MSRLOAD_ADDR_HIGH, 0);
-	asm_vmwrite (VMCS_VMENTRY_MSRLOAD_ADDR, 0);
-	asm_vmwrite (VMCS_VMENTRY_MSRLOAD_ADDR_HIGH, 0);
-	asm_vmwrite (VMCS_EXEC_VMCS_POINTER, 0);
-	asm_vmwrite (VMCS_EXEC_VMCS_POINTER_HIGH, 0);
-	asm_vmwrite (VMCS_TSC_OFFSET, 0);
-	asm_vmwrite (VMCS_TSC_OFFSET_HIGH, 0);
+	asm_vmwrite64 (VMCS_ADDR_IOBMP_A, 0xFFFFFFFFFFFFFFFFULL);
+	asm_vmwrite64 (VMCS_ADDR_IOBMP_B, 0xFFFFFFFFFFFFFFFFULL);
+	asm_vmwrite64 (VMCS_ADDR_MSRBMP, 0xFFFFFFFFFFFFFFFFULL);
+	asm_vmwrite64 (VMCS_VMEXIT_MSRSTORE_ADDR, 0);
+	asm_vmwrite64 (VMCS_VMEXIT_MSRLOAD_ADDR, 0);
+	asm_vmwrite64 (VMCS_VMENTRY_MSRLOAD_ADDR, 0);
+	asm_vmwrite64 (VMCS_EXEC_VMCS_POINTER, 0);
+	asm_vmwrite64 (VMCS_TSC_OFFSET, 0);
 	/* 64-Bit Guest-State Fields */
-	asm_vmwrite (VMCS_VMCS_LINK_POINTER, 0xFFFFFFFF);
-	asm_vmwrite (VMCS_VMCS_LINK_POINTER_HIGH, 0xFFFFFFFF);
-	asm_vmwrite (VMCS_GUEST_IA32_DEBUGCTL, 0);
-	asm_vmwrite (VMCS_GUEST_IA32_DEBUGCTL_HIGH, 0);
+	asm_vmwrite64 (VMCS_VMCS_LINK_POINTER, 0xFFFFFFFFFFFFFFFFULL);
+	asm_vmwrite64 (VMCS_GUEST_IA32_DEBUGCTL, 0);
+	if (current->u.vt.save_load_efer_enable)
+		asm_vmwrite64 (VMCS_GUEST_IA32_EFER, 0);
 	/* 32-Bit Control Fields */
 	asm_vmwrite (VMCS_PIN_BASED_VMEXEC_CTL,
-		     (VMCS_PIN_BASED_VMEXEC_CTL_EXINTEXIT_BIT |
+		     (/* VMCS_PIN_BASED_VMEXEC_CTL_EXINTEXIT_BIT */0 |
 		      VMCS_PIN_BASED_VMEXEC_CTL_NMIEXIT_BIT |
-		      VMCS_PIN_BASED_VMEXEC_CTL_VIRTNMIS_BIT |
 		      pinbased_ctls_or) & pinbased_ctls_and);
 	asm_vmwrite (VMCS_PROC_BASED_VMEXEC_CTL,
 		     (/* XXX: VMCS_PROC_BASED_VMEXEC_CTL_HLTEXIT_BIT */0 |
 		      VMCS_PROC_BASED_VMEXEC_CTL_INVLPGEXIT_BIT |
 		      VMCS_PROC_BASED_VMEXEC_CTL_UNCONDIOEXIT_BIT |
 		      VMCS_PROC_BASED_VMEXEC_CTL_USETSCOFF_BIT |
+		      (procbased_ctls2_and ?
+		       VMCS_PROC_BASED_VMEXEC_CTL_ACTIVATECTLS2_BIT : 0) |
 		      procbased_ctls_or) & procbased_ctls_and);
 	asm_vmwrite (VMCS_EXCEPTION_BMP, 0xFFFFFFFF);
 	asm_vmwrite (VMCS_PAGEFAULT_ERRCODE_MASK, 0);
 	asm_vmwrite (VMCS_PAGEFAULT_ERRCODE_MATCH, 0);
 	asm_vmwrite (VMCS_CR3_TARGET_COUNT, 0);
 	asm_vmwrite (VMCS_VMEXIT_CTL, (exit_ctls_or & exit_ctls_and) |
-		     exitctl64);
+		     exitctl64 | exitctl_efer);
 	asm_vmwrite (VMCS_VMEXIT_MSR_STORE_COUNT, 0);
 	asm_vmwrite (VMCS_VMEXIT_MSR_LOAD_COUNT, 0);
-	asm_vmwrite (VMCS_VMENTRY_CTL, entry_ctls_or & entry_ctls_and);
+	asm_vmwrite (VMCS_VMENTRY_CTL, (entry_ctls_or & entry_ctls_and) |
+		     entryctl_efer);
 	asm_vmwrite (VMCS_VMENTRY_MSR_LOAD_COUNT, 0);
 	asm_vmwrite (VMCS_VMENTRY_INTR_INFO_FIELD, 0);
 	asm_vmwrite (VMCS_VMENTRY_EXCEPTION_ERRCODE, 0);
 	asm_vmwrite (VMCS_VMENTRY_INSTRUCTION_LEN, 0);
 	asm_vmwrite (VMCS_TPR_THRESHOLD, 0);
+	if (procbased_ctls2_and)
+		asm_vmwrite (VMCS_PROC_BASED_VMEXEC_CTL2, procbased_ctls2);
 	/* 32-Bit Guest-State Fields */
 	asm_vmwrite (VMCS_GUEST_ES_LIMIT, guest_riv.es.limit);
 	asm_vmwrite (VMCS_GUEST_CS_LIMIT, guest_riv.cs.limit);
@@ -358,13 +410,17 @@ vt_vmcs_init (void)
 	asm_vmwrite (VMCS_HOST_IA32_SYSENTER_EIP, sysenter_eip);
 	asm_vmwrite (VMCS_HOST_RSP, 0xDEADBEEF);
 	asm_vmwrite (VMCS_HOST_RIP, 0xDEADBEEF);
+	if (current->u.vt.save_load_efer_enable) {
+		asm_rdmsr64 (MSR_IA32_EFER, &host_efer);
+		asm_vmwrite64 (VMCS_HOST_IA32_EFER, host_efer);
+	}
 
 	/* allocate iobmp */
 	vt_iopass_init ();
 }
 
 static void
-vt_realmode_data_init (void)
+vt__realmode_data_init (void)
 {
 	current->u.vt.realmode.idtr.base = 0;
 	current->u.vt.realmode.idtr.limit = 0x3FF;
@@ -373,28 +429,58 @@ vt_realmode_data_init (void)
 	current->u.vt.realmode.tr_base = 0;
 }
 
+static void
+vt__vmcs_exit (void)
+{
+	free_page (current->u.vt.vi.vmcs_region_virt);
+}
+
 void
 vt_vminit (void)
 {
-	vt_vmcs_init();
-	current->u.vt.vr.pe = 1;
-	current->u.vt.vr.pg = 1;
-	vt_realmode_data_init();
-	vt_msr_init();
-	vt_ept_init();
-	cpu_mmu_spt_init();
+	vt__vmcs_init ();
+	vt__realmode_data_init ();
+	vt_msr_init ();
+	vt_paging_init ();
+	call_initfunc ("vcpu");
 }
 
 void
 vt_vmexit (void)
 {
-	asm_vmxoff();
+	vt__vmcs_exit ();
 }
 
 void
 vt_init (void)
 {
-	vt_vmx_init ();
-	vt_vmxon ();
-	VT_DBG("VMX operation is enabled on CPU%d.\n", get_cpu_id());
+	vt__vmx_init ();
+	vt__vmxon ();
+}
+
+void
+vt_enable_resume (void)
+{
+	ASSERT (!current->u.vt.saved_vmcs);
+	alloc_page (&current->u.vt.saved_vmcs, NULL);
+	asm_vmclear (&current->u.vt.vi.vmcs_region_phys);
+	memcpy (current->u.vt.saved_vmcs, current->u.vt.vi.vmcs_region_virt,
+		PAGESIZE);
+	asm_vmptrld (&current->u.vt.vi.vmcs_region_phys);
+	current->u.vt.first = true;
+	spinlock_lock (&currentcpu->suspend_lock);
+}
+
+void
+vt_resume (void)
+{
+	ASSERT (current->u.vt.saved_vmcs);
+	vt__vmxon ();
+	memcpy (current->u.vt.vi.vmcs_region_virt, current->u.vt.saved_vmcs,
+		PAGESIZE);
+	asm_vmclear (&current->u.vt.vi.vmcs_region_phys);
+	asm_vmptrld (&current->u.vt.vi.vmcs_region_phys);
+	current->u.vt.first = true;
+	spinlock_init (&currentcpu->suspend_lock);
+	spinlock_lock (&currentcpu->suspend_lock);
 }
