@@ -26,20 +26,19 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  * POSSIBILITY OF SUCH DAMAGE.
  */
-/*
- * Copyright (c) 2010-2014 Yuichi Watanabe
- */
 
 #include "asm.h"
 #include "constants.h"
-#include "cpu_mmu_spt.h"
 #include "current.h"
 #include "entry.h"
 #include "mm.h"
 #include "panic.h"
-#include <core/printf.h>
+#include "printf.h"
 #include "seg.h"
-#include "vt_internal.h"
+#include "vt_main.h"
+#include "vt_msr.h"
+#include "vt_paging.h"
+#include "vt_regs.h"
 
 #define REALMODE_IDTR_BASE	0
 #define REALMODE_IDTR_LIMIT	0
@@ -47,8 +46,6 @@
 #define REALMODE_TR_ACR		0x8B
 #define REALMODE_TR_BASE	sym_to_phys (head)
 
-#define CR4_PASSTHROUGH CR4_OSXSAVE_BIT
-
 #define VMREAD(a,b) do { \
 	ulong tmp; \
  \
@@ -109,7 +106,7 @@ vt_read_general_reg (enum general_reg reg, ulong *val)
 		*val = current->u.vt.vr.r15;
 		break;
 	default:
-		panic ("Unknown register.");
+		panic ("Fatal error: unknown register.");
 	}
 }
 
@@ -166,7 +163,7 @@ vt_write_general_reg (enum general_reg reg, ulong val)
 		current->u.vt.vr.r15 = val;
 		break;
 	default:
-		panic ("Unknown register.");
+		panic ("Fatal error: unknown register.");
 	}
 }
 
@@ -187,7 +184,7 @@ vt_read_control_reg (enum control_reg reg, ulong *val)
 		asm_vmread (VMCS_CR4_READ_SHADOW, val);
 		break;
 	default:
-		panic ("Unknown control register.");
+		panic ("Fatal error: unknown control register.");
 	}
 }
 
@@ -234,106 +231,86 @@ pe_change_enable_sw (bool pe)
 
 	/* enable emulation */
 	vr->sw.num = 0; /* number of emulated instructions */
-	vr->sw.enable = 0;
+	vr->sw.enable
+		= SW_SREG_ES_BIT | SW_SREG_CS_BIT
+		| SW_SREG_SS_BIT | SW_SREG_DS_BIT
+		| SW_SREG_FS_BIT | SW_SREG_GS_BIT;
 
 	if (pe) {
 		/* real address mode to protected mode */
-
 		/* clear a virtual 8086 mode flag */
 		asm_vmread (VMCS_GUEST_RFLAGS, &rflags);
 		rflags &= ~RFLAGS_VM_BIT;
 		asm_vmwrite (VMCS_GUEST_RFLAGS, rflags);
-
-		/*
-		 * To keep segment base and limit in real mode,
-		 * we don't touch them.
-		 */
-
-		/*
-		 * Set segment access rights to proper value for protect mode.
-		 * The DPL of CS and SS must be zero.
-		 */
+		/* segment base and limit are the same as in real mode */
+		/* set segment selectors to 8 */
+		asm_vmwrite (VMCS_GUEST_ES_SEL, 8);
+		asm_vmwrite (VMCS_GUEST_CS_SEL, 8);
+		asm_vmwrite (VMCS_GUEST_SS_SEL, 8);
+		asm_vmwrite (VMCS_GUEST_DS_SEL, 8);
+		asm_vmwrite (VMCS_GUEST_FS_SEL, 8);
+		asm_vmwrite (VMCS_GUEST_GS_SEL, 8);
+		/* set segment access rights to proper value */
+		/* the DPL of CS and SS must be zero */
 		asm_vmwrite (VMCS_GUEST_ES_ACCESS_RIGHTS, 0xF3);
 		asm_vmwrite (VMCS_GUEST_CS_ACCESS_RIGHTS, 0x9B);
 		asm_vmwrite (VMCS_GUEST_SS_ACCESS_RIGHTS, 0x93);
 		asm_vmwrite (VMCS_GUEST_DS_ACCESS_RIGHTS, 0xF3);
 		asm_vmwrite (VMCS_GUEST_FS_ACCESS_RIGHTS, 0xF3);
 		asm_vmwrite (VMCS_GUEST_GS_ACCESS_RIGHTS, 0xF3);
-
-		asm_vmwrite (VMCS_GUEST_SS_SEL, 8);
-		/*
-		 * If a RPL of CS or SS is 0, it is usable as it is
-		 * without emulation. A null selector is also acceptable.
-		 * If a RPL is not 0, set segment selector to dummy value
-		 * (RPL 0, TI 0, Index 1), and emulate it.
-		 */
+		/* if a selector of CS or SS is a multiple of 4,
+		   it is usable as it is without emulation */
+		/* a null selector is also acceptable */
+		asm_vmwrite (VMCS_GUEST_ES_SEL, vr->sw.es);
+		vr->sw.enable &= ~SW_SREG_ES_BIT;
 		if ((vr->sw.cs & 3) == 0) {
 			asm_vmwrite (VMCS_GUEST_CS_SEL, vr->sw.cs);
-		} else {
-			vr->sw.enable |= SW_SREG_CS_BIT;
-			asm_vmwrite (VMCS_GUEST_CS_SEL, 8);
+			vr->sw.enable &= ~SW_SREG_CS_BIT;
 		}
-
 		if ((vr->sw.ss & 3) == 0) {
 			asm_vmwrite (VMCS_GUEST_SS_SEL, vr->sw.ss);
-		} else {
-			asm_vmwrite (VMCS_GUEST_CS_SEL, 8);
-			vr->sw.enable |= SW_SREG_SS_BIT;
+			vr->sw.enable &= ~SW_SREG_SS_BIT;
 		}
-
-		asm_vmwrite (VMCS_GUEST_ES_SEL, vr->sw.es);
 		asm_vmwrite (VMCS_GUEST_DS_SEL, vr->sw.ds);
+		vr->sw.enable &= ~SW_SREG_DS_BIT;
 		asm_vmwrite (VMCS_GUEST_FS_SEL, vr->sw.fs);
+		vr->sw.enable &= ~SW_SREG_FS_BIT;
 		asm_vmwrite (VMCS_GUEST_GS_SEL, vr->sw.gs);
+		vr->sw.enable &= ~SW_SREG_GS_BIT;
 	} else {
 		/* protected mode to real address mode */
-
 		/* set a virtual 8086 mode flag */
 		asm_vmread (VMCS_GUEST_RFLAGS, &rflags);
 		rflags |= RFLAGS_VM_BIT;
 		asm_vmwrite (VMCS_GUEST_RFLAGS, rflags);
-
-		/*
-		 * If the guest is in virtual-8086 mode, base-address
-		 * fields must be the selector field shifted left 4
-		 * bits.
-		 */
+		/* segment selector must be segment base / 16
+		   in virtual 8086 mode */
 		/* FIXME: segment base must be a multiple of 16.
 		   if not, it will be panic. */
-		asm_vmread (VMCS_GUEST_CS_BASE, &tmp);
-		asm_vmwrite (VMCS_GUEST_CS_SEL, tmp >> 4);
-		if ((tmp >> 4) != vr->sw.cs)
-		       vr->sw.enable |= SW_SREG_CS_BIT;
-
 		asm_vmread (VMCS_GUEST_ES_BASE, &tmp);
 		asm_vmwrite (VMCS_GUEST_ES_SEL, tmp >> 4);
-		if ((tmp >> 4) != vr->sw.es)
-			vr->sw.enable |= SW_SREG_ES_BIT;
-
+		if ((tmp >> 4) == vr->sw.es)
+			vr->sw.enable &= ~SW_SREG_ES_BIT;
+		asm_vmread (VMCS_GUEST_CS_BASE, &tmp);
+		asm_vmwrite (VMCS_GUEST_CS_SEL, tmp >> 4);
+		if ((tmp >> 4) == vr->sw.cs)
+			vr->sw.enable &= ~SW_SREG_CS_BIT;
 		asm_vmread (VMCS_GUEST_SS_BASE, &tmp);
 		asm_vmwrite (VMCS_GUEST_SS_SEL, tmp >> 4);
-		if ((tmp >> 4) != vr->sw.ss)
-			vr->sw.enable |= SW_SREG_SS_BIT;
-
+		if ((tmp >> 4) == vr->sw.ss)
+			vr->sw.enable &= ~SW_SREG_SS_BIT;
 		asm_vmread (VMCS_GUEST_DS_BASE, &tmp);
 		asm_vmwrite (VMCS_GUEST_DS_SEL, tmp >> 4);
-		if ((tmp >> 4) != vr->sw.ds)
-			vr->sw.enable |= SW_SREG_DS_BIT;
-
+		if ((tmp >> 4) == vr->sw.ds)
+			vr->sw.enable &= ~SW_SREG_DS_BIT;
 		asm_vmread (VMCS_GUEST_FS_BASE, &tmp);
 		asm_vmwrite (VMCS_GUEST_FS_SEL, tmp >> 4);
-		if ((tmp >> 4) != vr->sw.fs)
-			vr->sw.enable |= SW_SREG_FS_BIT;
-
+		if ((tmp >> 4) == vr->sw.fs)
+			vr->sw.enable &= ~SW_SREG_FS_BIT;
 		asm_vmread (VMCS_GUEST_GS_BASE, &tmp);
 		asm_vmwrite (VMCS_GUEST_GS_SEL, tmp >> 4);
-		if ((tmp >> 4) != vr->sw.gs)
-			vr->sw.enable |= SW_SREG_GS_BIT;
-
-		/*
-		 * If the guest will be virtual-8086, limit fields
-		 * must be 0000FFFFH.
-		 */
+		if ((tmp >> 4) == vr->sw.gs)
+			vr->sw.enable &= ~SW_SREG_GS_BIT;
 		/* set segment access rights to proper value */
 		asm_vmwrite (VMCS_GUEST_ES_ACCESS_RIGHTS, 0xF3);
 		asm_vmwrite (VMCS_GUEST_CS_ACCESS_RIGHTS, 0xF3);
@@ -341,10 +318,7 @@ pe_change_enable_sw (bool pe)
 		asm_vmwrite (VMCS_GUEST_DS_ACCESS_RIGHTS, 0xF3);
 		asm_vmwrite (VMCS_GUEST_FS_ACCESS_RIGHTS, 0xF3);
 		asm_vmwrite (VMCS_GUEST_GS_ACCESS_RIGHTS, 0xF3);
-		/*
-		 * If the guest will be virtual-8086, access-rights
-		 * fields must be 000000F3H
-		 */
+		/* set segment limit to 65535 */
 		asm_vmwrite (VMCS_GUEST_ES_LIMIT, 0xFFFF);
 		asm_vmwrite (VMCS_GUEST_CS_LIMIT, 0xFFFF);
 		asm_vmwrite (VMCS_GUEST_SS_LIMIT, 0xFFFF);
@@ -353,6 +327,7 @@ pe_change_enable_sw (bool pe)
 		asm_vmwrite (VMCS_GUEST_GS_LIMIT, 0xFFFF);
 	}
 
+	vt_update_exception_bmp ();
 }
 
 static void
@@ -363,7 +338,7 @@ pe_change (bool pe)
 	/* IDTR is different between real mode emulation and protected mode */
 	vt_read_idtr (&idtr_base, &idtr_limit);
 	vt_write_idtr (REALMODE_IDTR_BASE, REALMODE_IDTR_LIMIT);
-	current->u.vt.vr.pe = pe;
+	current->u.vt.vr.re = !pe;
 	vt_write_idtr (idtr_base, idtr_limit);
 
 	/* switching TR */
@@ -371,114 +346,47 @@ pe_change (bool pe)
 
 	/* enable emulation */
 	pe_change_enable_sw (pe);
+
+	current->u.vt.exint_re_pending = false;
+	current->u.vt.exint_update = true;
 }
 
 void
 vt_write_control_reg (enum control_reg reg, ulong val)
 {
-	ulong cr0_0, cr0_1;
-	ulong cr4_0, cr4_1;
-	ulong host_val;
-	ulong old_val;
-	ulong cr4;
-
 	switch (reg) {
 	case CONTROL_REG_CR0:
 		asm_vmwrite (VMCS_CR0_READ_SHADOW, val);
 		if (!(val & CR0_PE_BIT))
 			val &= ~CR0_PG_BIT;
-		if (!!(val & CR0_PE_BIT) != !!current->u.vt.vr.pe) {
+		if (!!(val & CR0_PE_BIT) != !current->u.vt.vr.re &&
+		    !current->u.vt.unrestricted_guest)
 			pe_change (!!(val & CR0_PE_BIT));
-		}
 		if (!!(val & CR0_PG_BIT) != !!current->u.vt.vr.pg) {
 			current->u.vt.vr.pg = !!(val & CR0_PG_BIT);
-			vt_ept_pg_change(current->u.vt.vr.pg);
-			/*
-			 * Update actual guest cr4.
-			 */
-			vt_read_control_reg(CONTROL_REG_CR4, &cr4);
-			vt_write_control_reg(CONTROL_REG_CR4, cr4);
+			vt_paging_pg_change ();
 		}
 		vt_msr_update_lma ();
-		if (!(current->u.vt.ept_enabled && current->u.vt.vr.pg)) {
-			val |= CR0_WP_BIT;
-		}
-		asm_rdmsr (MSR_IA32_VMX_CR0_FIXED0, &cr0_0);
-		asm_rdmsr (MSR_IA32_VMX_CR0_FIXED1, &cr0_1);
-		val &= cr0_1;
-		val |= cr0_0;
-		asm_vmwrite (VMCS_GUEST_CR0, val);
-		cpu_mmu_spt_updatecr3();
+		asm_vmwrite (VMCS_GUEST_CR0, vt_paging_apply_fixed_cr0 (val));
+		vt_paging_updatecr3 ();
+		vt_paging_flush_guest_tlb ();
 		break;
 	case CONTROL_REG_CR2:
 		current->u.vt.vr.cr2 = val;
 		break;
 	case CONTROL_REG_CR3:
-		current->u.vt.vr.cr3 = val & ~CR3_NO_INVALIDATE;
-		if (current->u.vt.ept_enabled && current->u.vt.vr.pg) {
-			asm_vmwrite(VMCS_GUEST_CR3, val & ~CR3_NO_INVALIDATE);
-		} else {
-			cpu_mmu_spt_updatecr3();
-		}
-		vt_read_control_reg(CONTROL_REG_CR3, &cr4);
-		if ((cr4 & CR4_PCIDE_BIT) == 0) {
-			vt_invalidate_tlb();
-		} else if ((val & CR3_NO_INVALIDATE) == 0) {
-			/*
-			 * We can't specify PCID with invept.
-			 * So all TLB will be invalidated with any PCIID.
-			 */
-			vt_invalidate_tlb();
-		}
+		current->u.vt.vr.cr3 = val;
+		vt_paging_updatecr3 ();
+		vt_paging_flush_guest_tlb ();
 		break;
 	case CONTROL_REG_CR4:
-		/*
-		 *Update host cr4.
-		 */
-		asm_rdcr4(&host_val);
-		host_val = (host_val & ~CR4_PASSTHROUGH) /* Only OSXSAVE */
-			| (val & CR4_PASSTHROUGH);
-		asm_wrcr4(host_val);
-		asm_vmwrite(VMCS_HOST_CR4, host_val);
-
-		/*
-		 * Update guest cr4 shadow.
-		 */
-		asm_vmread (VMCS_CR4_READ_SHADOW, &old_val);
 		asm_vmwrite (VMCS_CR4_READ_SHADOW, val);
-
-		/*
-		 * Update acutual guest cr4.
-		 */
-		asm_rdmsr (MSR_IA32_VMX_CR4_FIXED0, &cr4_0);
-		asm_rdmsr (MSR_IA32_VMX_CR4_FIXED1, &cr4_1);
-		val &= cr4_1;
-		val |= cr4_0;
-		if (!(current->u.vt.ept_enabled && current->u.vt.vr.pg)) {
-			/*
-			 * Shadow paging requires PAE for both 32 bit
-			 * and 64 bit guest.
-			 */
-			val |= CR4_PAE_BIT;
-		}
-		if (!current->u.vt.vr.pg) {
-			/*
-			 * For non-paged protect mode, disable
-			 * supervisor-mode execution prevention,
-			 * because PTE_US_BIT in shadow pte is set,
-			 * and guest software runs in supervisor-mode.
-			 */
-			val &= ~CR4_SMEP_BIT;
-		}
-		asm_vmwrite(VMCS_GUEST_CR4, val);
-
-		if (old_val ^ val) {
-			vt_invalidate_tlb();
-			cpu_mmu_spt_updatecr3();
-		}
+		asm_vmwrite (VMCS_GUEST_CR4, vt_paging_apply_fixed_cr4 (val));
+		vt_paging_updatecr3 ();
+		vt_paging_flush_guest_tlb ();
 		break;
 	default:
-		panic ("Unknown control register.");
+		panic ("Fatal error: unknown control register.");
 	}
 }
 
@@ -531,7 +439,7 @@ vt_read_sreg_sel (enum sreg s, u16 *val)
 		*val = tmp;
 		break;
 	default:
-		panic ("Unknown sreg.");
+		panic ("Fatal error: unknown sreg.");
 	}
 }
 
@@ -558,7 +466,7 @@ vt_read_sreg_acr (enum sreg s, ulong *val)
 		asm_vmread (VMCS_GUEST_GS_ACCESS_RIGHTS, val);
 		break;
 	default:
-		panic ("Unknown sreg.");
+		panic ("Fatal error: unknown sreg.");
 	}
 }
 
@@ -603,7 +511,7 @@ vt_read_sreg_base (enum sreg s, ulong *val)
 		asm_vmread (VMCS_GUEST_GS_BASE, val);
 		break;
 	default:
-		panic ("Unknown sreg.");
+		panic ("Fatal error: unknown sreg.");
 	}
 }
 
@@ -639,19 +547,10 @@ vt_read_sreg_limit (enum sreg s, ulong *val)
 		asm_vmread (VMCS_GUEST_GS_LIMIT, val);
 		break;
 	default:
-		panic ("Unknown sreg.");
+		panic ("Fatal error: unknown sreg.");
 	}
 }
 
-void
-vt_spt_setcr3 (ulong cr3)
-{
-	if (current->u.vt.ept_enabled && current->u.vt.vr.pg) {
-		return;
-	}
-	asm_vmwrite (VMCS_GUEST_CR3, cr3);
-}
-
 void
 vt_read_ip (ulong *val)
 {
@@ -669,14 +568,14 @@ void
 vt_read_flags (ulong *val)
 {
 	asm_vmread (VMCS_GUEST_RFLAGS, val);
-	if (!current->u.vt.vr.pe)
+	if (current->u.vt.vr.re)
 		*val &= ~RFLAGS_VM_BIT;
 }
 
 void
 vt_write_flags (ulong val)
 {
-	if (!current->u.vt.vr.pe)
+	if (current->u.vt.vr.re)
 		val |= RFLAGS_VM_BIT;
 	asm_vmwrite (VMCS_GUEST_RFLAGS, val);
 }
@@ -698,7 +597,7 @@ vt_write_gdtr (ulong base, ulong limit)
 void
 vt_read_idtr (ulong *base, ulong *limit)
 {
-	if (current->u.vt.vr.pe) {
+	if (!current->u.vt.vr.re) {
 		asm_vmread (VMCS_GUEST_IDTR_BASE, base);
 		asm_vmread (VMCS_GUEST_IDTR_LIMIT, limit);
 	} else {
@@ -710,7 +609,7 @@ vt_read_idtr (ulong *base, ulong *limit)
 void
 vt_write_idtr (ulong base, ulong limit)
 {
-	if (current->u.vt.vr.pe) {
+	if (!current->u.vt.vr.re) {
 		asm_vmwrite (VMCS_GUEST_IDTR_BASE, base);
 		asm_vmwrite (VMCS_GUEST_IDTR_LIMIT, limit);
 	} else {
@@ -763,6 +662,52 @@ vt_get_current_regs_in_vmcs (struct regs_in_vmcs *p)
 	asm_rdrflags (&p->rflags);
 }
 
+void
+vt_get_vmcs_regs_in_vmcs (struct regs_in_vmcs *p)
+{
+	VMREAD (VMCS_GUEST_ES_SEL, &p->es.sel);
+	VMREAD (VMCS_GUEST_CS_SEL, &p->cs.sel);
+	VMREAD (VMCS_GUEST_SS_SEL, &p->ss.sel);
+	VMREAD (VMCS_GUEST_DS_SEL, &p->ds.sel);
+	VMREAD (VMCS_GUEST_FS_SEL, &p->fs.sel);
+	VMREAD (VMCS_GUEST_GS_SEL, &p->gs.sel);
+	VMREAD (VMCS_GUEST_LDTR_SEL, &p->ldtr.sel);
+	VMREAD (VMCS_GUEST_TR_SEL, &p->tr.sel);
+	VMREAD (VMCS_GUEST_CR0, &p->cr0);
+	VMREAD (VMCS_GUEST_CR3, &p->cr3);
+	VMREAD (VMCS_GUEST_CR4, &p->cr4);
+	VMREAD (VMCS_GUEST_ES_LIMIT, &p->es.limit);
+	VMREAD (VMCS_GUEST_CS_LIMIT, &p->cs.limit);
+	VMREAD (VMCS_GUEST_SS_LIMIT, &p->ss.limit);
+	VMREAD (VMCS_GUEST_DS_LIMIT, &p->ds.limit);
+	VMREAD (VMCS_GUEST_FS_LIMIT, &p->fs.limit);
+	VMREAD (VMCS_GUEST_GS_LIMIT, &p->gs.limit);
+	VMREAD (VMCS_GUEST_LDTR_LIMIT, &p->ldtr.limit);
+	VMREAD (VMCS_GUEST_TR_LIMIT, &p->tr.limit);
+	VMREAD (VMCS_GUEST_GDTR_LIMIT, &p->gdtr.limit);
+	VMREAD (VMCS_GUEST_IDTR_LIMIT, &p->idtr.limit);
+	VMREAD (VMCS_GUEST_ES_ACCESS_RIGHTS, &p->es.acr);
+	VMREAD (VMCS_GUEST_CS_ACCESS_RIGHTS, &p->cs.acr);
+	VMREAD (VMCS_GUEST_SS_ACCESS_RIGHTS, &p->ss.acr);
+	VMREAD (VMCS_GUEST_DS_ACCESS_RIGHTS, &p->ds.acr);
+	VMREAD (VMCS_GUEST_FS_ACCESS_RIGHTS, &p->fs.acr);
+	VMREAD (VMCS_GUEST_GS_ACCESS_RIGHTS, &p->gs.acr);
+	VMREAD (VMCS_GUEST_LDTR_ACCESS_RIGHTS, &p->ldtr.acr);
+	VMREAD (VMCS_GUEST_TR_ACCESS_RIGHTS, &p->tr.acr);
+	VMREAD (VMCS_GUEST_ES_BASE, &p->es.base);
+	VMREAD (VMCS_GUEST_CS_BASE, &p->cs.base);
+	VMREAD (VMCS_GUEST_SS_BASE, &p->ss.base);
+	VMREAD (VMCS_GUEST_DS_BASE, &p->ds.base);
+	VMREAD (VMCS_GUEST_FS_BASE, &p->fs.base);
+	VMREAD (VMCS_GUEST_GS_BASE, &p->gs.base);
+	VMREAD (VMCS_GUEST_LDTR_BASE, &p->ldtr.base);
+	VMREAD (VMCS_GUEST_TR_BASE, &p->tr.base);
+	VMREAD (VMCS_GUEST_GDTR_BASE, &p->gdtr.base);
+	VMREAD (VMCS_GUEST_IDTR_BASE, &p->idtr.base);
+	VMREAD (VMCS_GUEST_DR7, &p->dr7);
+	VMREAD (VMCS_GUEST_RFLAGS, &p->rflags);
+}
+
 void
 vt_write_realmode_seg (enum sreg s, u16 val)
 {
@@ -770,49 +715,40 @@ vt_write_realmode_seg (enum sreg s, u16 val)
 	case SREG_ES:
 		asm_vmwrite (VMCS_GUEST_ES_SEL, val);
 		asm_vmwrite (VMCS_GUEST_ES_BASE, val << 4);
-		asm_vmwrite (VMCS_GUEST_ES_ACCESS_RIGHTS, 0xF3);
-		asm_vmwrite (VMCS_GUEST_ES_LIMIT, 0xFFFF);
+		current->u.vt.vr.sw.enable &= ~SW_SREG_ES_BIT;
 		break;
 	case SREG_CS:
 		asm_vmwrite (VMCS_GUEST_CS_SEL, val);
 		asm_vmwrite (VMCS_GUEST_CS_BASE, val << 4);
-		/*
-		 * In virtual 8086 mode, access-right field for CS
-		 *  segment must be 000000F3H
-		 */
-		asm_vmwrite (VMCS_GUEST_CS_ACCESS_RIGHTS, 0xF3);
-		asm_vmwrite (VMCS_GUEST_CS_LIMIT, 0xFFFF);
+		current->u.vt.vr.sw.enable &= ~SW_SREG_CS_BIT;
 		break;
 	case SREG_SS:
 		asm_vmwrite (VMCS_GUEST_SS_SEL, val);
 		asm_vmwrite (VMCS_GUEST_SS_BASE, val << 4);
-		asm_vmwrite (VMCS_GUEST_SS_ACCESS_RIGHTS, 0xF3);
-		asm_vmwrite (VMCS_GUEST_SS_LIMIT, 0xFFFF);
+		current->u.vt.vr.sw.enable &= ~SW_SREG_SS_BIT;
 		break;
 	case SREG_DS:
 		asm_vmwrite (VMCS_GUEST_DS_SEL, val);
 		asm_vmwrite (VMCS_GUEST_DS_BASE, val << 4);
-		asm_vmwrite (VMCS_GUEST_DS_ACCESS_RIGHTS, 0xF3);
-		asm_vmwrite (VMCS_GUEST_DS_LIMIT, 0xFFFF);
+		current->u.vt.vr.sw.enable &= ~SW_SREG_DS_BIT;
 		break;
 	case SREG_FS:
 		asm_vmwrite (VMCS_GUEST_FS_SEL, val);
 		asm_vmwrite (VMCS_GUEST_FS_BASE, val << 4);
-		asm_vmwrite (VMCS_GUEST_FS_ACCESS_RIGHTS, 0xF3);
-		asm_vmwrite (VMCS_GUEST_FS_LIMIT, 0xFFFF);
+		current->u.vt.vr.sw.enable &= ~SW_SREG_FS_BIT;
 		break;
 	case SREG_GS:
 		asm_vmwrite (VMCS_GUEST_GS_SEL, val);
 		asm_vmwrite (VMCS_GUEST_GS_BASE, val << 4);
-		asm_vmwrite (VMCS_GUEST_GS_ACCESS_RIGHTS, 0xF3);
-		asm_vmwrite (VMCS_GUEST_GS_LIMIT, 0xFFFF);
+		current->u.vt.vr.sw.enable &= ~SW_SREG_GS_BIT;
 		break;
 	default:
-		panic ("Unknown sreg.");
+		panic ("Fatal error: unknown sreg.");
 	}
+	vt_update_exception_bmp ();
 }
 
-vmmerr_t
+enum vmmerr
 vt_writing_sreg (enum sreg s)
 {
 	switch (s) {
@@ -837,5 +773,86 @@ vt_writing_sreg (enum sreg s)
 	case SREG_DEFAULT:
 		return VMMERR_AVOID_COMPILER_WARNING;
 	}
+	vt_update_exception_bmp ();
 	return VMMERR_SW;
 }
+
+void
+vt_reset (void)
+{
+	asm_vmwrite (VMCS_CR0_READ_SHADOW, CR0_ET_BIT);
+	asm_vmwrite (VMCS_GUEST_CR0, vt_paging_apply_fixed_cr0 (CR0_ET_BIT));
+	current->u.vt.vr.cr2 = 0;
+	current->u.vt.vr.cr3 = 0;
+	asm_vmwrite (VMCS_CR4_READ_SHADOW, 0);
+	asm_vmwrite (VMCS_GUEST_CR4, vt_paging_apply_fixed_cr4 (0));
+	vt_write_msr (MSR_IA32_EFER, 0);
+	asm_vmwrite (VMCS_GUEST_ES_SEL, 0);
+	asm_vmwrite (VMCS_GUEST_CS_SEL, 0);
+	asm_vmwrite (VMCS_GUEST_SS_SEL, 0);
+	asm_vmwrite (VMCS_GUEST_DS_SEL, 0);
+	asm_vmwrite (VMCS_GUEST_FS_SEL, 0);
+	asm_vmwrite (VMCS_GUEST_GS_SEL, 0);
+	asm_vmwrite (VMCS_GUEST_ES_BASE, 0);
+	asm_vmwrite (VMCS_GUEST_CS_BASE, 0);
+	asm_vmwrite (VMCS_GUEST_SS_BASE, 0);
+	asm_vmwrite (VMCS_GUEST_DS_BASE, 0);
+	asm_vmwrite (VMCS_GUEST_FS_BASE, 0);
+	asm_vmwrite (VMCS_GUEST_GS_BASE, 0);
+	if (current->u.vt.unrestricted_guest) {
+		asm_vmwrite (VMCS_GUEST_ES_ACCESS_RIGHTS, 0x93);
+		asm_vmwrite (VMCS_GUEST_CS_ACCESS_RIGHTS, 0x93);
+		asm_vmwrite (VMCS_GUEST_SS_ACCESS_RIGHTS, 0x93);
+		asm_vmwrite (VMCS_GUEST_DS_ACCESS_RIGHTS, 0x93);
+		asm_vmwrite (VMCS_GUEST_FS_ACCESS_RIGHTS, 0x93);
+		asm_vmwrite (VMCS_GUEST_GS_ACCESS_RIGHTS, 0x93);
+	} else {
+		asm_vmwrite (VMCS_GUEST_ES_ACCESS_RIGHTS, 0xF3);
+		asm_vmwrite (VMCS_GUEST_CS_ACCESS_RIGHTS, 0xF3);
+		asm_vmwrite (VMCS_GUEST_SS_ACCESS_RIGHTS, 0xF3);
+		asm_vmwrite (VMCS_GUEST_DS_ACCESS_RIGHTS, 0xF3);
+		asm_vmwrite (VMCS_GUEST_FS_ACCESS_RIGHTS, 0xF3);
+		asm_vmwrite (VMCS_GUEST_GS_ACCESS_RIGHTS, 0xF3);
+	}
+	asm_vmwrite (VMCS_GUEST_ES_LIMIT, 0xFFFF);
+	asm_vmwrite (VMCS_GUEST_CS_LIMIT, 0xFFFF);
+	asm_vmwrite (VMCS_GUEST_SS_LIMIT, 0xFFFF);
+	asm_vmwrite (VMCS_GUEST_DS_LIMIT, 0xFFFF);
+	asm_vmwrite (VMCS_GUEST_FS_LIMIT, 0xFFFF);
+	asm_vmwrite (VMCS_GUEST_GS_LIMIT, 0xFFFF);
+	if (current->u.vt.unrestricted_guest) {
+		asm_vmwrite (VMCS_GUEST_RFLAGS, RFLAGS_ALWAYS1_BIT);
+		asm_vmwrite (VMCS_GUEST_IDTR_BASE, 0);
+		asm_vmwrite (VMCS_GUEST_IDTR_LIMIT, 0xFFFF);
+		asm_vmwrite (VMCS_GUEST_TR_LIMIT, 0xFFFF);
+		asm_vmwrite (VMCS_GUEST_TR_ACCESS_RIGHTS, 0x8B);
+		asm_vmwrite (VMCS_GUEST_TR_BASE, 0);
+		current->u.vt.vr.re = 0;
+	} else {
+		asm_vmwrite (VMCS_GUEST_RFLAGS, RFLAGS_ALWAYS1_BIT |
+			     RFLAGS_VM_BIT | RFLAGS_IOPL_0);
+		asm_vmwrite (VMCS_GUEST_IDTR_BASE, REALMODE_IDTR_BASE);
+		asm_vmwrite (VMCS_GUEST_IDTR_LIMIT, REALMODE_IDTR_LIMIT);
+		asm_vmwrite (VMCS_GUEST_TR_LIMIT, REALMODE_TR_LIMIT);
+		asm_vmwrite (VMCS_GUEST_TR_ACCESS_RIGHTS, REALMODE_TR_ACR);
+		asm_vmwrite (VMCS_GUEST_TR_BASE, REALMODE_TR_BASE);
+		current->u.vt.vr.re = 1;
+	}
+	if (current->u.vt.vr.pg) {
+		current->u.vt.vr.pg = 0;
+		vt_paging_pg_change ();
+	}
+	current->u.vt.vr.sw.enable = 0;
+	current->u.vt.lme = 0;
+	current->u.vt.realmode.tr_limit = 0xFFFF;
+	current->u.vt.realmode.tr_acr = 0x8B;
+	current->u.vt.realmode.tr_base = 0;
+	current->u.vt.realmode.idtr.base = 0;
+	current->u.vt.realmode.idtr.limit = 0xFFFF;
+	current->u.vt.exint_re_pending = false;
+	current->u.vt.exint_update = true;
+	vt_msr_update_lma ();
+	vt_paging_updatecr3 ();
+	vt_paging_flush_guest_tlb ();
+	vt_update_exception_bmp ();
+}
