@@ -26,9 +26,6 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  * POSSIBILITY OF SUCH DAMAGE.
  */
-/*
- * Copyright (c) 2010-2012 Yuichi Watanabe
- */
 
 #ifndef __CORE_STRING_H
 #define __CORE_STRING_H
@@ -37,6 +34,65 @@
 
 #define USE_BUILTIN_STRING
 
+static inline void *
+memset_slow (void *addr, int val, int len)
+{
+	char *p;
+
+	p = addr;
+	while (len--)
+		*p++ = val;
+	return addr;
+}
+
+static inline void *
+memcpy_slow (void *dest, void *src, int len)
+{
+	char *p, *q;
+
+	p = dest;
+	q = src;
+	while (len--)
+		*p++ = *q++;
+	return dest;
+}
+
+static inline int
+strcmp_slow (char *s1, char *s2)
+{
+	int r, c1, c2;
+
+	do {
+		c1 = *s1++;
+		c2 = *s2++;
+		r = c1 - c2;
+	} while (!r && c1);
+	return r;
+}
+
+static inline int
+memcmp_slow (void *p1, void *p2, int len)
+{
+	int r, i;
+	char *q1, *q2;
+
+	q1 = p1;
+	q2 = p2;
+	for (r = 0, i = 0; !r && i < len; i++)
+		r = *q1++ - *q2++;
+	return r;
+}
+
+static inline int
+strlen_slow (char *p)
+{
+	int len = 0;
+
+	while (*p++)
+		len++;
+	return len;
+}
+
 #ifdef USE_BUILTIN_STRING
 #	define memset(addr, val, len)	memset_builtin (addr, val, len)
 #	define memcpy(dest, src, len)	memcpy_builtin (dest, src, len)
@@ -81,179 +137,21 @@ strlen_builtin (const char *p)
 {
 	return __builtin_strlen (p);
 }
-
-#else /* USE_BUILTIN_STRING */
-
-static inline void *
-memset_slow (void *addr, int val, int len)
-{
-	char *p;
-
-	p = addr;
-	while (len--)
-		*p++ = val;
-	return addr;
-}
-
-static inline void *
-memcpy_slow (void *dest, const void *src, int len)
-{
-	char *p, *q;
-
-	p = dest;
-	q = src;
-	while (len-- > 0)
-		*p++ = *q++;
-	return dest;
-}
-
-static inline int
-strcmp_slow (const char *s1, const char *s2)
-{
-	int r, c1, c2;
-
-	do {
-		c1 = *s1++;
-		c2 = *s2++;
-		r = c1 - c2;
-	} while (!r && c1);
-	return r;
-}
-
-static inline int
-memcmp_slow (const void *p1, const void *p2, int len)
-{
-	int r, i;
-	char *q1, *q2;
-
-	q1 = p1;
-	q2 = p2;
-	for (r = 0, i = 0; !r && i < len; i++)
-		r = *q1++ - *q2++;
-	return r;
-}
-
-static inline int
-strlen_slow (const char *p)
-{
-	int len = 0;
-
-	while (*p++)
-		len++;
-	return len;
-}
 #endif /* USE_BUILTIN_STRING */
 
+
 int strncmp(register const char *s1, register const char *s2, register size_t n);
 char *strncpy(char * __restrict dst, const char * __restrict src, size_t n);
 char *strcpy(char * __restrict to, const char * __restrict from);
 char *strncat(char * __restrict dst, const char * __restrict src, size_t n);
 
-/*
- * From NetBSD src/external/bsd/libpcap/dist/pcap-int.h
- */
-/*
- * Copyright (c) 1998 Todd C. Miller <Todd.Miller (at) courtesan.com>
- * All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. The name of the author may not be used to endorse or promote products
- *    derived from this software without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
- * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
- * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
- * THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
- * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
- * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
- * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
- * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
- * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
- * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- * Id: compat.c,v 1.11 2011/09/27 08:22:55 plunky Exp
- * $NetBSD$
- */
 
 /*
- * From NetBSD src/crypto/dist/ipsec-tools/src/racoon/misc.h
+ * From NetBSD 6.1.3 usr/src/external/bsd/libpcap/dist/pcap-int.h
  */
-/*-
- * Copyright (c) 1997, 2002 The NetBSD Foundation, Inc.
- * All rights reserved.
- *
- * This code is derived from software contributed to The NetBSD Foundation
- * by Klaus Klein and Jason R. Thorpe.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- *
- * THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
- * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
- * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR CONTRIBUTORS
- * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- *
- *	NetBSD: basename.c,v 1.9 2009/11/24 13:34:20 tnozaki Exp
- */
-
-/*
- * Copyright (c) 1987, 1993
- * The Regents of the University of California.  All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- *
- *	NetBSD: gettemp.c,v 1.13 2003/12/05 00:57:36 uebayasi Exp
- */
-#define strlcpy(x, y, z) \
-	(strncpy((x), (y), (z)),		  \
-	 ((z) <= 0 ? 0 : ((x)[(z) - 1] = '\0')),  \
-	 strlen((y)))
-
 /*
  * Copyright (c) 1994, 1995, 1996
- * The Regents of the University of California.  All rights reserved.
+ *	The Regents of the University of California.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -265,8 +163,8 @@ char *strncat(char * __restrict dst, const char * __restrict src, size_t n);
  *    documentation and/or other materials provided with the distribution.
  * 3. All advertising materials mentioning features or use of this software
  *    must display the following acknowledgement:
- *    This product includes software developed by the Computer Systems
- *    Engineering Group at Lawrence Berkeley Laboratory.
+ *	This product includes software developed by the Computer Systems
+ *	Engineering Group at Lawrence Berkeley Laboratory.
  * 4. Neither the name of the University nor of the Laboratory may be used
  *    to endorse or promote products derived from this software without
  *    specific prior written permission.
@@ -283,8 +181,46 @@ char *strncat(char * __restrict dst, const char * __restrict src, size_t n);
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * @(#) Header: /tcpdump/master/libpcap/pcap-int.h,v 1.94 2008-09-16 00:20:23 guy Exp  (LBL)
+ * @(#) Header: /tcpdump/master/libpcap/pcap-int.h,v 1.94 2008-09-16 00:20:23 guy Exp (LBL)
  */
+
+#define strlcpy(x, y, z) \
+	(strncpy((x), (y), (z)),		  \
+	 ((z) <= 0 ? 0 : ((x)[(z) - 1] = '\0')),  \
+	 strlen((y)))
+
+/*
+ * From NetBSD 6.1.3 usr/src/crypto/dist/ipsec-tools/src/racoon/misc.h
+ */
+/*
+ * Copyright (C) 1995, 1996, 1997, and 1998 WIDE Project.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3. Neither the name of the project nor the names of its contributors
+ *    may be used to endorse or promote products derived from this software
+ *    without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
+ * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+ * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+ * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+ * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
+ */
+
 #define strlcat(d,s,l) strncat(d,s,(l)-strlen(d)-1)
 
 #endif
