@@ -50,7 +50,7 @@ void *vm_get_driver_data(drvdata_hdl_t handle);
 void *vm_get_driver_data_of_vm(char *name, drvdata_hdl_t handle);
 apic_id_t vm_vbsp_apic_id(void);
 void vm_iommu_enabled(void);
-void vm_shutdown(void);
+void vm_shutdown(u8 state);
 vmmerr_t vm_reset(void);
 
 #endif /* __CORE_VM_H */
